require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe CostTypesController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) do
        {
          code: FFaker::Name.name,
          name: FFaker::Name.name,
          fields: [{ "name": 'amount', "type": 'float' }]
        }
      end

      let(:invalid_attributes) do
        {
          code: nil,
          name: FFaker::Name.name,
          fields: [{ "name": 'amount', "type": 'float' }]
        }
      end

      describe 'GET #index' do
        it 'returns a success response' do
          create(:cost_type)
          get :index, params: {}
          expect(response).to be_successful
          result = JSON(response.body)
          expect(result['data'].count).to eq 1
        end
      end

      describe 'GET #show' do
        it 'returns a success response' do
          cost_type = create(:cost_type)
          get :show, params: { id: cost_type.to_param }
          expect(response).to be_successful
          result = JSON(response.body)
          expect(result['data']['id']).to eq cost_type.id
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new CostType' do
            expect do
              post :create, params: { cost_type: valid_attributes }
            end.to change(CostType, :count).by(1)
          end

          it 'renders a JSON response with the new cost_type' do
            post :create, params: { cost_type: valid_attributes }
            expect(response).to have_http_status(:created)
            expect(response.content_type).to eq('application/json')
            expect(response.location).to eq(cost_type_url(CostType.last))
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the new cost_type' do
            post :create, params: { cost_type: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          let(:new_attributes) do
            {
              name: FFaker::Name.name
            }
          end

          it 'updates the requested cost_type' do
            cost_type = create(:cost_type)
            put :update, params: { id: cost_type.to_param, cost_type: new_attributes }
            cost_type.reload
            expect(cost_type.name).to eq new_attributes[:name]
          end

          it 'renders a JSON response with the cost_type' do
            cost_type = create(:cost_type)
            put :update, params: { id: cost_type.to_param, cost_type: valid_attributes }
            expect(response).to have_http_status(:ok)
            expect(response.content_type).to eq('application/json')
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the cost_type' do
            cost_type = create(:cost_type)
            put :update, params: { id: cost_type.to_param, cost_type: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end
    end
  end
end