require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe JobCardsController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        @u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: @u.full_name, email: @u.email, id: @u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) do
        {
          maintenance_service_order_id: create(:maintenance_service_order).id,
          code: FFaker::Name.name,
          start_date: Date.new,
          due_date: Date.new + 10,
          service_provider_id: create(:equipment_location).id,
          prepared_by_id: create(:user).id
        }
      end

      let(:invalid_attributes) do
        {
          maintenance_service_order_id: nil,
          code: FFaker::Name.name,
          start_date: Date.new,
          due_date: Date.new + 10,
          service_provider_id: create(:equipment_location).id
        }
      end

      describe 'GET #index' do
        it 'returns a success response' do
          create(:job_card)
          get :index
          expect(response).to be_successful
        end

        it 'filters cards of the current user only' do
          2.times { create(:job_card, prepared_by: @u) }
          2.times { create(:job_card) }
          get :index
          result = JSON(response.body)
          expect(result['data'].count).to eq 2
        end
      end

      describe 'GET #show' do
        it 'returns a success response' do
          job_card = create(:job_card)
          get :show, params: { id: job_card.to_param }
          expect(response).to be_successful
        end
      end

      describe 'GET #checkouts' do
        it 'filters checkouts to current user' do
          2.times { create(:job_card) }
          u = create(:user)
          2.times { create(:job_card, checkout_by_id: u.id) }

          token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
          request.headers['Authorization'] = "Bearer #{token}"

          get :checkouts
          result = JSON(response.body)
          expect(result['data'].count).to eq 2
        end
      end

      describe 'GET #checkins' do
        it 'filters checkins to current user' do
          2.times { create(:job_card) }
          u = create(:user)
          2.times { create(:job_card, checkin_by_id: u.id) }

          token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
          request.headers['Authorization'] = "Bearer #{token}"

          get :checkins
          result = JSON(response.body)
          expect(result['data'].count).to eq 2
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new JobCard' do
            expect do
              post :create, params: { job_card: valid_attributes }
            end.to change(JobCard, :count).by(1)
          end

          it 'renders a JSON response with the new job_card' do

            post :create, params: { job_card: valid_attributes }
            expect(response).to have_http_status(:created)
            expect(response.content_type).to eq('application/json')
            expect(response.location).to eq(job_card_url(JobCard.last))
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the new job_card' do
            post :create, params: { job_card: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          let(:new_attributes) do
            {
              start_date: Date.new
            }
          end

          it 'updates the requested job_card' do
            job_card = create(:job_card)
            put :update, params: { id: job_card.to_param, job_card: new_attributes }
            job_card.reload
            expect(job_card.start_date).to eq new_attributes[:start_date]
          end

          it 'renders a JSON response with the job_card' do
            job_card = create(:job_card)
            put :update, params: { id: job_card.to_param, job_card: valid_attributes }
            expect(response).to have_http_status(:ok)
            expect(response.content_type).to eq('application/json')
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the job_card' do
            job_card = create(:job_card)
            put :update, params: { id: job_card.to_param, job_card: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

      describe 'PUT #checkout' do
        it 'updates checkout data' do
          job_card = create(:job_card)
          u1 = create(:user)
          l1 = create(:equipment_location)

          params = {
            checkout_by_id: u1.id,
            checkout_date: Date.new,
            checkout_to_id: l1.id
          }

          put :checkout, params: { id: job_card.id, checkout: params }
          result = JSON(response.body)

          expect(result['success']).to be_truthy
          job_card.reload
          expect(job_card.checkout_by).to eq u1
          expect(job_card.checkout_to).to eq l1
        end
      end

      describe 'PUT #checkin' do
        it 'updates checkin data' do
          job_card = create(:job_card)
          u1 = create(:user)
          l1 = create(:equipment_location)

          params = {
            checkin_by_id: u1.id,
            checkin_date: Date.new,
            checkin_to_id: l1.id
          }

          put :checkin, params: { id: job_card.id, checkin: params }
          result = JSON(response.body)

          expect(result['success']).to be_truthy
          job_card.reload
          expect(job_card.checkin_by).to eq u1
          expect(job_card.checkin_to).to eq l1
        end
      end
    end
  end
end