require 'rails_helper'


module Mks
  module Maintenance
    RSpec.describe MaintenanceCostsController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) do
        {
          job_card_id: create(:job_card).id,
          cost_type_id: create(:cost_type).id,
          cost_detail: { "part_number": 'ABCDEF', "quantity": 1, "price": 200 },
          remark: FFaker::Name.name
        }
      end

      let(:invalid_attributes) do
        {
          job_card_id: nil,
          cost_type_id: create(:cost_type).id,
          cost_detail: { "part_number": 'ABCDEF', "quantity": 1, "price": 200 },
          remark: FFaker::Name.name
        }
      end

      describe 'GET #index' do
        it 'returns a success response' do
          job_card = create(:job_card)
          create(:maintenance_cost, job_card: job_card)
          get :index, params: { id: job_card.id }
          expect(response).to be_successful
        end
      end

      describe 'GET #show' do
        it 'returns a success response' do
          maintenance_cost = create(:maintenance_cost)
          get :show, params: { id: maintenance_cost.to_param }
          expect(response).to be_successful
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new MaintenanceCost' do
            expect do
              post :create, params: { maintenance_cost: valid_attributes }
            end.to change(MaintenanceCost, :count).by(1)
          end

          it 'renders a JSON response with the new maintenance_cost' do
            post :create, params: { maintenance_cost: valid_attributes }
            expect(response).to have_http_status(:created)
            expect(response.content_type).to eq('application/json')
            expect(response.location).to eq(maintenance_cost_url(MaintenanceCost.last))
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the new maintenance_cost' do
            post :create, params: { maintenance_cost: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          let(:new_attributes) do
            {
              remark: FFaker::Name.name
            }
          end

          it 'updates the requested maintenance_cost' do
            maintenance_cost = create(:maintenance_cost)
            put :update, params: { id: maintenance_cost.to_param, maintenance_cost: new_attributes }
            maintenance_cost.reload
            expect(maintenance_cost.remark).to eq new_attributes[:remark]
          end

          it 'renders a JSON response with the maintenance_cost' do
            maintenance_cost = create(:maintenance_cost)
            put :update, params: { id: maintenance_cost.to_param, maintenance_cost: valid_attributes }
            expect(response).to have_http_status(:ok)
            expect(response.content_type).to eq('application/json')
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the maintenance_cost' do
            maintenance_cost = create(:maintenance_cost)
            put :update, params: { id: maintenance_cost.to_param, maintenance_cost: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end
    end
  end
end