require 'rails_helper'

module Mks
  module Maintenance
    RSpec.describe MaintenanceServiceOrdersController, type: :controller do
      routes { Mks::Maintenance::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"
      end

      let(:valid_attributes) {
        {
          title: FFaker::Name.name,
          description: FFaker::Name.name,
          start_date: Date.today,
          end_date: Date.today,
          remark: FFaker::Name.name,
          current_reading: 100,
          approved: true,
          equipment_item_id: create(:equipment_item).id,
          maintenance_type_id: create(:maintenance_type).id,
          prepared_by_id: create(:user).id,
          approved_by_id: create(:user).id
        }
      }

      let(:invalid_attributes) {
        {
          title: nil,
          description: nil,
          start_date: nil,
          end_date: nil,
          remark: FFaker::Name.name,
          current_reading: 100,
          approved: true,
          equipment_item_id: create(:equipment_item).id,
          maintenance_type_id: create(:maintenance_type).id,
          prepared_by_id: create(:user),
          approved_by_id: create(:user)
        }
      }

      describe 'GET #index' do
        it 'returns all maintenance services order' do
          3.times { create(:maintenance_service_order) }
          get :index
          result = JSON(response.body)
          expect(result['data'].count).to eq 3
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new MaintenanceServiceOrder' do
            expect {
              post :create, params: { maintenance_service_order: valid_attributes }, format: :json
            }.to change(MaintenanceServiceOrder, :count).by(1)
          end
          it 'returns a success message' do
            post :create, params: { maintenance_service_order: valid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance services order saved successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            post :create, params: { maintenance_service_order: invalid_attributes }, format: :json
            result = JSON(response.body)
            expect(result['title'].count).to be > 0
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          it 'updates the requested maintenance services order' do
            mso = create(:maintenance_service_order)
            put :update, params: { id: mso.id, maintenance_service_order: { title: 'Oil Change' } }, format: :json
            mso.reload
            expect(mso.title).to eq('Oil Change')
          end

          it 'returns a success message' do
            mso = create(:maintenance_service_order)
            put :update, params: { id: mso.id, maintenance_service_order: { title: 'Oil Change' } }, format: :json
            result = JSON(response.body)
            expect(result['message']).to eq('Maintenance services order updated successfully !')
          end
        end
        context 'with invalid params' do
          it 'returns error message(s)' do
            mso = create(:maintenance_service_order)
            put :update, params: { id: mso.id, maintenance_service_order: { title: nil } }, format: :json
            result = JSON(response.body)
            expect(result['title'].count).to be > 0
          end
        end
      end

      describe 'GET #filter' do
        it 'returns filtered maintenance service orders' do
          3.times { create(:maintenance_service_order) }
          2.times { create(:maintenance_service_order, approved: true) }
          filters = [{key: 'approved', value: true, cond: 'P'}]
          get :filter, params: { type: 'Mks::Maintenance::MaintenanceServiceOrder', filters: filters}
          result = JSON(response.body)
          expect(result['data'].count).to eq(2)
        end
      end
    end
  end
end